/*
 * Licensed Materials - Property of IBM
 *
 * trousers - An open source TCG Software Stack
 *
 * (C) Copyright International Business Machines Corp. 2004, 2005
 *
 */


#ifndef WIN32
#include <unistd.h>
#endif
#include <fcntl.h>
#ifdef WIN32
#include <ws2tcpip.h>
#include <windows.h>
#include "libc_interface.h"
#endif
#include <stdio.h>
#include <errno.h>
#include <string.h>
#ifndef WIN32
#include <sys/ioctl.h>
#endif

#include "trousers/tss.h"
#include "trousers_types.h"
#include "linux/tpm.h"
#include "tcslog.h"
#include "tddl.h"

#ifndef WIN32
struct tpm_device_node tpm_device_nodes[] = {
	{"/dev/tpm0", TDDL_UNDEF, TDDL_UNDEF},
	{"/udev/tpm0", TDDL_UNDEF, TDDL_UNDEF},
	{"/dev/tpm", TDDL_UNDEF, TDDL_UNDEF},
	{NULL, 0, 0}
};

struct tpm_device_node *opened_device = NULL;
#else
struct tpm_device_node opened_device;
int tddli_dh = -1;
#endif

BYTE txBuffer[TDDL_TXBUF_SIZE];
TSS_BOOL use_in_socket = FALSE;
struct tcsd_config *_tcsd_options = NULL;

#ifndef WIN32
#include <sys/socket.h>
#include <sys/un.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <netdb.h>
#include <fcntl.h>


int
open_device()
{
	int i = 0, fd = -1, tcp_device_port;
	char *tcp_device_hostname = NULL;
	char *un_socket_device_path = NULL;
	char *tcp_device_port_string = NULL;
	
	if (getenv("TCSD_USE_TCP_DEVICE")) {
		if ((tcp_device_hostname = getenv("TCSD_TCP_DEVICE_HOSTNAME")) == NULL)
			tcp_device_hostname = "localhost";
		if ((un_socket_device_path = getenv("TCSD_UN_SOCKET_DEVICE_PATH")) == NULL)
			un_socket_device_path = "/var/run/tpm/tpmd_socket:0";
		if ((tcp_device_port_string = getenv("TCSD_TCP_DEVICE_PORT")) != NULL)
			tcp_device_port = atoi(tcp_device_port_string);
		else
			tcp_device_port = 6545;
	 
		
		fd = socket(AF_INET, SOCK_STREAM, 0);
		if (fd > 0) {
			struct hostent *host = gethostbyname(tcp_device_hostname);
			if (host != NULL) {   
				struct sockaddr_in addr;
				memset(&addr, 0x0, sizeof(addr));
				addr.sin_family = host->h_addrtype;
				addr.sin_port   = htons(tcp_device_port);
				memcpy(&addr.sin_addr,
						host->h_addr,
						host->h_length);
				if (connect(fd,	(struct sockaddr *)&addr,
					    sizeof(addr)) < 0) {
					close(fd);
					fd = -1;
				} else
					use_in_socket = TRUE;
			} else {
				close (fd);
				fd = -1;
			}
		}
	       
		if (fd < 0) {
			struct sockaddr_un addr;

			fd = socket(AF_UNIX, SOCK_STREAM, 0);
			if (fd >= 0) {
				addr.sun_family = AF_UNIX;
				strncpy(addr.sun_path, un_socket_device_path,
						sizeof(addr.sun_path));
				if (connect(fd, (void *)&addr, sizeof(addr)) < 0) {
					close(fd);
					fd = -1;
				}
			}
		}
	} 
	
	if (fd < 0) {
		/* tpm_device_paths is filled out in tddl.h */
		for (i = 0; tpm_device_nodes[i].path != NULL; i++) {
			errno = 0;
			if ((fd = open(tpm_device_nodes[i].path, O_RDWR)) >= 0)
				break;
		}
	}
	
	if (fd > 0) {
		opened_device = &(tpm_device_nodes[i]);
		tpm_device_nodes[i].fd = fd;
	}
	return fd;
}
#else
TSS_RESULT open_device(const char *device_name)
{
	/* open the named pipe and generate a posix file handle */
	DWORD mode = PIPE_TYPE_MESSAGE | PIPE_READMODE_MESSAGE;
	HANDLE ph = CreateFile(device_name, GENERIC_READ | GENERIC_WRITE,
				0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	SetNamedPipeHandleState(ph, &mode, NULL, NULL);
	tddli_dh = _open_osfhandle((DWORD)ph, O_RDWR | O_BINARY);
	if (tddli_dh < 0)
		return TDDLERR(TDDL_E_FAIL);

	return TSS_SUCCESS;
}

TSS_RESULT send_to_tpm(BYTE* pTransmitBuf, UINT32 TransmitBufLen)
{
	ssize_t res;
	res = write(tddli_dh, pTransmitBuf, TransmitBufLen);
	if (res < 0 || (UINT32)res != TransmitBufLen) 
		return TDDL_E_IOERROR;

	return TSS_SUCCESS;
}

TSS_RESULT receive_from_tpm(BYTE* pReceiveBuf, UINT32* puntReceiveBufLen)
{
	ssize_t res;
	UINT32 len;
	if (*puntReceiveBufLen < 10)
		return TDDL_E_INSUFFICIENT_BUFFER;
	res = read(tddli_dh, pReceiveBuf, *puntReceiveBufLen);
	if (res < 10)
		return TDDL_E_IOERROR;
	*puntReceiveBufLen = res;
	len = ((UINT32)pReceiveBuf[2] << 24) | ((UINT32)pReceiveBuf[3] << 16)
	      | ((UINT32)pReceiveBuf[4] << 8) | (UINT32)pReceiveBuf[5];
	if (len != *puntReceiveBufLen)
		return TDDL_E_INSUFFICIENT_BUFFER;

	return TSS_SUCCESS;
}
#endif

TSS_RESULT
Tddli_Open()
{
#ifdef WIN32
	HRESULT hr;
	LPTSTR tpm_device_name = TEXT("\\\\.\\pipe\\tpmd:0"); 

	if (opened_device.hContext != 0 || tddli_dh != -1) {
		LogDebug("attempted to re-open the TPM driver!");
		return TDDLERR(TDDL_E_ALREADY_OPENED);
	}

	opened_device.contextParams.version = TBS_CONTEXT_VERSION_ONE;
	hr = Tbsi_Context_Create(&opened_device.contextParams,
				 &opened_device.hContext);
	if (FAILED(hr)) {
		LogDebug("Failed to contact TPM Base Services, trying to connect to TPM Emulator...");
		hr = open_device(tpm_device_name);
		if (hr != TSS_SUCCESS) {
			LogError("Failed to find an available TPM!");
			return TDDLERR(TDDL_E_FAIL);
		}
	}
#else
	int rc;

	if (opened_device != NULL) {
		LogDebug("attempted to re-open the TPM driver!");
		return TDDLERR(TDDL_E_ALREADY_OPENED);
	}

	rc = open_device();
	if (rc < 0) {
		LogError("Could not find a device to open!");
		if (errno == ENOENT) {
			/* File DNE */
			return TDDLERR(TDDL_E_COMPONENT_NOT_FOUND);
		}

		return TDDLERR(TDDL_E_FAIL);
	}

#endif
	return TSS_SUCCESS;
}

TSS_RESULT
Tddli_Close()
{
#ifdef WIN32
	if (opened_device.hContext == 0 && tddli_dh == -1) {
		LogDebug("attempted to re-close the TPM driver!");
		return TDDLERR(TDDL_E_ALREADY_CLOSED);
	}

	if (opened_device.hContext != 0) {
		Tbsip_Context_Close(opened_device.hContext);
		opened_device.hContext = 0;
	} else {
		close(tddli_dh);
		tddli_dh = -1;
	}
#else	
	if (opened_device == NULL) {
		LogDebug("attempted to re-close the TPM driver!");
		return TDDLERR(TDDL_E_ALREADY_CLOSED);
	}

	close(opened_device->fd);
	opened_device->fd = TDDL_UNDEF;
	opened_device = NULL;

#endif
	return TSS_SUCCESS;
}

TSS_RESULT
Tddli_TransmitData(BYTE * pTransmitBuf, UINT32 TransmitBufLen, BYTE * pReceiveBuf,
		   UINT32 * pReceiveBufLen)
{
	int sizeResult;
#ifdef WIN32
	HRESULT hr;

	if (TransmitBufLen > TDDL_TXBUF_SIZE) {
		LogError("buffer size handed to TDDL is too large! (%u bytes)", TransmitBufLen);
		return TDDLERR(TDDL_E_FAIL);
	}

	memcpy(txBuffer, pTransmitBuf, TransmitBufLen);
	sizeResult = sizeof(txBuffer);
	LogDebug("Calling write to driver");

	if (opened_device.hContext != 0) {
		hr = Tbsip_Submit_Command(opened_device.hContext,
								  TBS_COMMAND_LOCALITY_ZERO,
								  TBS_COMMAND_PRIORITY_NORMAL,
								  txBuffer,
								  TransmitBufLen,
								  txBuffer,
								  (unsigned *)&sizeResult);
		if (FAILED(hr)) {
			return TDDLERR(TDDL_E_FAIL);
		}

		if ((unsigned)sizeResult > *pReceiveBufLen) {
			return TDDLERR(TDDL_E_INSUFFICIENT_BUFFER);
		}
	} else if (tddli_dh != -1) {
		hr = send_to_tpm(txBuffer, TransmitBufLen);
		if (hr != TSS_SUCCESS)
			return TDDLERR(TDDL_E_FAIL);
		
		hr = receive_from_tpm(txBuffer, (unsigned *)&sizeResult);
		if (hr != TSS_SUCCESS)
			return TDDLERR(TDDL_E_FAIL);
	}
#else
	if (TransmitBufLen > TDDL_TXBUF_SIZE) {
		LogError("buffer size handed to TDDL is too large! (%u bytes)", TransmitBufLen);
		return TDDLERR(TDDL_E_FAIL);
	}

	memcpy(txBuffer, pTransmitBuf, TransmitBufLen);
	LogDebug("Calling write to driver");

	if (use_in_socket) {
		Tddli_Close();
		if (Tddli_Open())
			return TDDLERR(TDDL_E_IOERROR);
	}

	switch (opened_device->transmit) {
		case TDDL_UNDEF:
			/* fall through */
		case TDDL_TRANSMIT_IOCTL:
			errno = 0;
			if ((sizeResult = ioctl(opened_device->fd, TPMIOC_TRANSMIT, txBuffer)) != -1) {
				opened_device->transmit = TDDL_TRANSMIT_IOCTL;
				break;
			}
			LogWarn("ioctl: (%d) %s", errno, strerror(errno));
			LogInfo("Falling back to Read/Write device support.");
			/* fall through */
		case TDDL_TRANSMIT_RW:
			if ((sizeResult = write(opened_device->fd,
						txBuffer,
						TransmitBufLen)) == (int)TransmitBufLen) {
				opened_device->transmit = TDDL_TRANSMIT_RW;
				sizeResult = read(opened_device->fd, txBuffer,
						  TDDL_TXBUF_SIZE);
				break;
			} else {
				if (sizeResult == -1) {
					LogError("write to device %s failed: %s",
						 opened_device->path,
						 strerror(errno));
				} else {
					LogError("wrote %d bytes to %s (tried "
						 "to write %d)", sizeResult,
						 opened_device->path,
						 TransmitBufLen);
				}
			}
			/* fall through */
		default:
			return TDDLERR(TDDL_E_IOERROR);
	}

	if (sizeResult < 0) {
		LogError("read from device %s failed: %s", opened_device->path, strerror(errno));
		return TDDLERR(TDDL_E_IOERROR);
	} else if (sizeResult == 0) {
		LogError("Zero bytes read from device %s", opened_device->path);
		return TDDLERR(TDDL_E_IOERROR);
	}

	if ((unsigned)sizeResult > *pReceiveBufLen) {
		LogError("read %d bytes from device %s, (only room for %d)", sizeResult,
				opened_device->path, *pReceiveBufLen);
		return TDDLERR(TDDL_E_INSUFFICIENT_BUFFER);
	}

#endif
	*pReceiveBufLen = sizeResult;

	memcpy(pReceiveBuf, txBuffer, *pReceiveBufLen);
	return TSS_SUCCESS;
}

TSS_RESULT
Tddli_GetStatus(UINT32 ReqStatusType, UINT32 *pStatus)
{
	return TDDLERR(TSS_E_NOTIMPL);
}

TSS_RESULT
Tddli_SetCapability(UINT32 CapArea, UINT32 SubCap,
		    BYTE *pSetCapBuf, UINT32 SetCapBufLen)
{
	return TDDLERR(TSS_E_NOTIMPL);
}

#ifdef WIN32
static TSS_RESULT cap_version(UINT32 SubCap, BYTE* pCapBuf,
                              UINT32* puntCapBufLen)
{
	TSS_RESULT res;
	UINT32 len = 18;
	BYTE buf[18];

	switch (SubCap) {
	case TDDL_CAP_VER_DRV:
		if (*puntCapBufLen < 4) return TDDL_E_INSUFFICIENT_BUFFER;
		*puntCapBufLen = 4;
		memcpy(pCapBuf, "\x01\x05\x00\x00", 4);
		return TSS_SUCCESS;

	case TDDL_CAP_VER_FW:
		if (*puntCapBufLen < 4) return TDDL_E_INSUFFICIENT_BUFFER;
		*puntCapBufLen = 4;
		res = send_to_tpm((BYTE *)"\x00\xc1\x00\x00\x00\x12\x00\x00\x00\x65"
		  "\x00\x00\x00\x06\x00\x00\x00\x00", 18);
		if (res != TSS_SUCCESS)
			return res;
		res = receive_from_tpm(buf, &len);
		if (res != TSS_SUCCESS)
			return res;
		if (len != 18 || (buf[6] | buf[7] | buf[8] | buf[9]) != 0)
			return TDDL_E_FAIL;
		memcpy(pCapBuf, &buf[14], 4);
		return TSS_SUCCESS;

	case TDDL_CAP_VER_FW_DATE:
		/* this is not yet supported by the TPM emulator */
		return TDDLERR(TSS_E_NOTIMPL);

	default:
		return TDDLERR(TSS_E_BAD_PARAMETER);
	}
}

static TSS_RESULT cap_property(UINT32 SubCap, BYTE* pCapBuf,
                               UINT32* puntCapBufLen)
{
	static const char *manufacturer = "Mario Strasser, ETH Zurich";
	static const char *type = "Software-based TPM Emulator";

	switch (SubCap) {
	case TDDL_CAP_PROP_MANUFACTURER:
		if (*puntCapBufLen < strlen(manufacturer))
			return TDDL_E_INSUFFICIENT_BUFFER;
		*puntCapBufLen = strlen(manufacturer);
		memcpy(pCapBuf, manufacturer, *puntCapBufLen);
		return TSS_SUCCESS;

	case TDDL_CAP_PROP_MODULE_TYPE:
		if (*puntCapBufLen < strlen(type)) 
			return TDDL_E_INSUFFICIENT_BUFFER;
		*puntCapBufLen = strlen(type);
		memcpy(pCapBuf, type, *puntCapBufLen);
		return TSS_SUCCESS;

	default:
		return TDDLERR(TSS_E_BAD_PARAMETER);
	}
}
#endif

TSS_RESULT
Tddli_GetCapability(UINT32 CapArea, UINT32 SubCap,
		    BYTE *pCapBuf, UINT32 *pCapBufLen)
{
#ifdef WIN32
	TSS_RESULT res = TSS_SUCCESS;
	if (tddli_dh < 0) 
		return TDDL_E_FAIL;
	switch (CapArea) {
	case TDDL_CAP_VERSION:
		res = cap_version(SubCap, pCapBuf, pCapBufLen);    
		break;
	case TDDL_CAP_PROPERTY:
		res = cap_property(SubCap, pCapBuf, pCapBufLen);
		break;
	default:
		res = TDDLERR(TSS_E_BAD_PARAMETER);
	}
	return res;
#else
	return TDDLERR(TSS_E_NOTIMPL);
#endif
}

TSS_RESULT Tddli_Cancel(void)
{
#ifdef WIN32
	return TSS_E_NOTIMPL;
#else
	int rc;

	if (opened_device->transmit == TDDL_TRANSMIT_IOCTL) {
		if ((rc = ioctl(opened_device->fd, TPMIOC_CANCEL, NULL)) == -1) {
			LogError("ioctl: (%d) %s", errno, strerror(errno));
			return TDDLERR(TDDL_E_FAIL);
		} else if (rc == -EIO) {
			/* The driver timed out while trying to tell the chip to cancel */
			return TDDLERR(TDDL_E_COMMAND_COMPLETED);
		}

		return TSS_SUCCESS;
	} else {
		return TDDLERR(TSS_E_NOTIMPL);
	}
#endif
}
