// test.cpp : Defines the entry point for the console application.
//

#include <stdio.h>
#include <string.h>
#include <fcntl.h>

#include <stdlib.h>
#include <trousers/tss.h>
#include <trousers/trousers.h>

#ifdef WIN32
#include <windows.h>
#include <sys/stat.h>
#else
#include <sys/stat.h>
#endif

#define ERR 1

#ifdef RAND
int main (int argc, char ** argv) {

    TSS_HCONTEXT        hContext;
    TSS_RESULT          result;
    TSS_HTPM            hTPM;
    BYTE                *rand;
	int i, j;
    
    TSS_UUID SRK_UUID = TSS_UUID_SRK;

    result = Tspi_Context_Create( &hContext );
    if ( result != TSS_SUCCESS )
    {
        printf( "ERR - Tspi_Context_Create");
        Tspi_Context_FreeMemory( hContext, NULL );
        Tspi_Context_Close( hContext );
        exit (ERR);
    }

    result = Tspi_Context_Connect( hContext, NULL );
    if ( result != TSS_SUCCESS )
    {
        printf( "ERR - Tspi_Context_Connect");
        Tspi_Context_FreeMemory( hContext, NULL );
        Tspi_Context_Close( hContext );
        exit (ERR);
    }
    
    result = Tspi_Context_GetTpmObject( hContext, &hTPM );
    if ( result != TSS_SUCCESS )
    {
        printf( "ERR - Tspi_Context_GetTpmObject");
        Tspi_Context_FreeMemory( hContext, NULL );
        Tspi_Context_Close( hContext );
        exit (ERR);
    }

    for( i=0; i<2; i++ )
    {
        for( j=0; j<6; j++ )
        {
            result = Tspi_TPM_GetRandom( hTPM, 1, &rand );
            if ( result != TSS_SUCCESS )
            {
                printf( "ERR - Tspi_Context_GetTpmObject");
                Tspi_Context_FreeMemory( hContext, NULL );
                Tspi_Context_Close( hContext );
                exit (ERR);
            }
            printf("%d ", (*rand%90)+1 );
        }
        printf("\n");    
    }
        
    /* Clean */
	Tspi_Context_CloseObject( hContext, hTPM );
    Tspi_Context_FreeMemory( hContext, NULL );
    Tspi_Context_Close( hContext );

    return 0;
}
#endif


#ifdef CREATEKEY
int main(int argc, char **argv)
{
	TSS_HCONTEXT hContext;
	TSS_HPOLICY  hPolicySRK ;
	TSS_HKEY     hSrk;
	TSS_HKEY     hKey;	
	TSS_RESULT   result;
	char srkpwd[50];
	int srkpwdlen= 0;
	TSS_UUID SRK_UUID=TSS_UUID_SRK;
	BYTE *keyblob = NULL;
	UINT32 keylen = 0;
	int fp;
	TSS_FLAG keyFlags = TSS_KEY_TYPE_STORAGE | TSS_KEY_SIZE_2048
			| TSS_KEY_VOLATILE | TSS_KEY_AUTHORIZATION | TSS_KEY_NOT_MIGRATABLE;
	
	result = Tspi_Context_Create(&hContext);
	if (result != TSS_SUCCESS) {
		printf("Error: Tspi_Context_Create returned %d\n", result);
		exit(result);
	}
	
	result = Tspi_Context_Connect(hContext,NULL);
	if (result != TSS_SUCCESS) {
		printf("Error: Tspi_Context_Connect returned %d)\n", result);
		Tspi_Context_FreeMemory(hContext, NULL);                 
		Tspi_Context_Close(hContext);
		exit(result);
	}	
	
	if((result = Tspi_Context_LoadKeyByUUID(hContext, TSS_PS_TYPE_SYSTEM, SRK_UUID, &hSrk)) != 0) {
		printf("LoadKeyByUUID(): res: %d\n", result);
		Tspi_Context_FreeMemory(hContext, NULL);
		Tspi_Context_Close(hContext);
		exit(result);
	}
	
	result = Tspi_GetPolicyObject(hSrk, TSS_POLICY_USAGE, &hPolicySRK);
	if (result != TSS_SUCCESS)
	{
		printf("Error: Tspi_GetPolicyObject returned %d\n", result);
		Tspi_Context_FreeMemory(hContext, NULL);
		Tspi_Context_Close(hContext);
		exit(result);
	}
		
	/* Set password */	
	printf("Insert the TPM SRK password: ");
	scanf("%s", srkpwd);
	srkpwdlen = strlen(srkpwd);

	result = Tspi_Policy_SetSecret(hPolicySRK, TSS_SECRET_MODE_PLAIN, srkpwdlen, (BYTE *)srkpwd);
	if (result != TSS_SUCCESS)
	{
		printf("Error: Tspi_Policy_SetSecret returned %d\n", result);
		Tspi_Context_FreeMemory(hContext, NULL);
		Tspi_Context_Close(hContext);
		exit(result);
	}
	
	result = Tspi_Context_CreateObject(hContext,TSS_OBJECT_TYPE_RSAKEY, keyFlags, &hKey); 
	if (result != 0)
	{
		printf("Error: Tspi_GetPolicyObject returned %d\n", result);
		Tspi_Context_FreeMemory(hContext, NULL);
		Tspi_Context_Close(hContext);
		exit(result);
	}
	
/*	if((result = Tspi_SetAttribUint32(hKey, TSS_TSPATTRIB_KEY_INFO, TSS_TSPATTRIB_KEYINFO_USAGE, TSS_KEYUSAGE_BIND))!= 0) {
		printf("Tspi_SetAttribUint32(): res: %d\n", result);
		Tspi_Context_FreeMemory(hContext, NULL);
		Tspi_Context_Close(hContext);
		exit(result);
	}*/
	
	if((result = Tspi_Key_CreateKey(hKey, hSrk, 0))!= 0) {
		printf("CreateKey(): res: %d\n", result);
		Tspi_Context_FreeMemory(hContext, NULL);
		Tspi_Context_Close(hContext);
		exit(result);
	}	
	
	printf("Result: %d\n", result);
	Tspi_GetAttribData(hKey, TSS_TSPATTRIB_KEY_BLOB, TSS_TSPATTRIB_KEYBLOB_BLOB, &keylen, &keyblob);
#ifdef WIN32
	if((fp=open("key", O_WRONLY | O_CREAT | O_BINARY, 0600))<0)
#else
	if((fp=open("key", O_WRONLY | O_CREAT, 0600))<0)
#endif
		goto end_create;

	write(fp, (char *)keyblob, keylen);
	close(fp);

end_create:
	Tspi_Key_UnloadKey(hKey);
	Tspi_Key_UnloadKey(hSrk);
	Tspi_Context_CloseObject(hContext,hKey);
	Tspi_Context_CloseObject(hContext,hSrk);
	Tspi_Context_CloseObject(hContext,hPolicySRK);
	Tspi_Context_Close(hContext);
	
	return result;
}

#endif

#ifdef LOADKEY

int createkey2(TSS_HCONTEXT hContext, TSS_HKEY hParent)
{
	TSS_RESULT   result;
	char srkpwd[50];
	int srkpwdlen= 0;
	TSS_UUID SRK_UUID=TSS_UUID_SRK;
	BYTE *keyblob = NULL;
	UINT32 keylen = 0;
	int fp;
	TSS_HKEY hKey;
	
	TSS_FLAG keyFlags = TSS_KEY_TYPE_BIND | TSS_KEY_SIZE_2048  | \
									 TSS_KEY_VOLATILE | TSS_KEY_AUTHORIZATION  | \
									 TSS_KEY_NOT_MIGRATABLE;

	
		
	/* Set password */	
/*	printf("Insert the TPM SRK password: ");
	scanf("%s", srkpwd);
	srkpwdlen = strlen(srkpwd);

	result = Tspi_Policy_SetSecret(hPolicySRK, TSS_SECRET_MODE_PLAIN, srkpwdlen, (BYTE *)srkpwd);
	if (result != TSS_SUCCESS)
	{
		printf("Error: Tspi_Policy_SetSecret returned %d\n", result);
		Tspi_Context_FreeMemory(hContext, NULL);
		Tspi_Context_Close(hContext);
		exit(result);
	}
*/	
	result = Tspi_Context_CreateObject(hContext,TSS_OBJECT_TYPE_RSAKEY, keyFlags, &hKey); 
	if (result != 0)
	{
		printf("Error: Tspi_GetPolicyObject returned %d(%s)\n", result, Trspi_Error_String(result));
		Tspi_Context_FreeMemory(hContext, NULL);
		Tspi_Context_Close(hContext);
		exit(result);
	}
	
/*	if((result = Tspi_SetAttribUint32(hKey, TSS_TSPATTRIB_KEY_INFO, TSS_TSPATTRIB_KEYINFO_USAGE, TSS_KEYUSAGE_BIND))!= 0) {
		printf("Tspi_SetAttribUint32(): res: %d\n", result);
		Tspi_Context_FreeMemory(hContext, NULL);
		Tspi_Context_Close(hContext);
		exit(result);
	}*/
	
	if((result = Tspi_Key_CreateKey(hKey, hParent, 0))!= 0) {
		printf("CreateKey(): res: %d(%s)\n", result, Trspi_Error_String(result));
		Tspi_Context_FreeMemory(hContext, NULL);
		Tspi_Context_Close(hContext);
		exit(result);
	}	
	
	printf("Result: %d(%s)\n", result, Trspi_Error_String(result));
	Tspi_GetAttribData(hKey, TSS_TSPATTRIB_KEY_BLOB, TSS_TSPATTRIB_KEYBLOB_BLOB, &keylen, &keyblob);
#ifdef WIN32
	if((fp=open("key2", O_WRONLY | O_CREAT | O_BINARY, 0600))<0)
#else
	if((fp=open("key2", O_WRONLY | O_CREAT, 0600))<0)
#endif
		goto end_create;

	write(fp, (char *)keyblob, keylen);
	close(fp);

end_create:
	Tspi_Key_UnloadKey(hKey);
	Tspi_Context_CloseObject(hContext,hKey);

	return result;
}

int main(int argc, char **argv)
{
	TSS_HCONTEXT hContext;
	TSS_HPOLICY  hPolicySRK ;
	TSS_HKEY     hSrk;
	TSS_HKEY     hKey;	
	TSS_RESULT   result;
	char srkpwd[50];
	int srkpwdlen= 0;
	TSS_UUID SRK_UUID=TSS_UUID_SRK;
	BYTE *keyblob = NULL;
	UINT32 keylen = 0;
	int fp;
	struct stat st;
	
	result = Tspi_Context_Create(&hContext);
	if (result != TSS_SUCCESS) {
		printf("Error: Tspi_Context_Create returned %d(%s)\n", result, Trspi_Error_String(result));
		exit(result);
	}
	
	result = Tspi_Context_Connect(hContext,NULL);
	if (result != TSS_SUCCESS) {
		printf("Error: Tspi_Context_Connect returned %d(%s)\n", result, Trspi_Error_String(result));
		Tspi_Context_FreeMemory(hContext, NULL);                 
		Tspi_Context_Close(hContext);
		exit(result);
	}	
	
	if((result = Tspi_Context_LoadKeyByUUID(hContext, TSS_PS_TYPE_SYSTEM, SRK_UUID, &hSrk)) != 0) {
		printf("LoadKeyByUUID(): res: %d\n", result);
		Tspi_Context_FreeMemory(hContext, NULL);
		Tspi_Context_Close(hContext);
		exit(result);
	}
	
	result = Tspi_GetPolicyObject(hSrk, TSS_POLICY_USAGE, &hPolicySRK);
	if (result != TSS_SUCCESS)
	{
		printf("Error: Tspi_GetPolicyObject returned %d(%s)\n", result, Trspi_Error_String(result));
		Tspi_Context_FreeMemory(hContext, NULL);
		Tspi_Context_Close(hContext);
		exit(result);
	}
		
	/* Set password */	
	printf("Insert the TPM SRK password: ");
	scanf("%s", srkpwd);
	srkpwdlen = strlen(srkpwd);

	result = Tspi_Policy_SetSecret(hPolicySRK, TSS_SECRET_MODE_PLAIN, srkpwdlen, (BYTE *)srkpwd);
	if (result != TSS_SUCCESS)
	{
		printf("Error: Tspi_Policy_SetSecret returned %d(%s)\n", result, Trspi_Error_String(result));
		Tspi_Context_FreeMemory(hContext, NULL);
		Tspi_Context_Close(hContext);
		exit(result);
	}

	stat("key", &st);
	keylen = st.st_size;
	keyblob=calloc(keylen, sizeof(BYTE));
#ifdef WIN32	
	if((fp=open("key", O_RDONLY | O_BINARY))<0)
#else
	if((fp=open("key", O_RDONLY))<0)
#endif
		goto end_load;

	read(fp, (char *)keyblob, keylen);
	close(fp);

	if((result=Tspi_Context_LoadKeyByBlob(hContext, hSrk, keylen, keyblob, &hKey))!= 0) {
		printf("LoadKeyByBlob(): res: %d(%s)\n", result), Trspi_Error_String(result);
		goto end_load;
	}

	printf("Result: %d(%s)\n", result, Trspi_Error_String(result));

	createkey2(hContext, hKey);

end_load:
	Tspi_Key_UnloadKey(hKey);
	Tspi_Key_UnloadKey(hSrk);
	Tspi_Context_CloseObject(hContext,hKey);
	Tspi_Context_CloseObject(hContext,hSrk);
	Tspi_Context_CloseObject(hContext,hPolicySRK);
	Tspi_Context_Close(hContext);
	
	return result;



}
#endif

#ifdef TAKE_OWNERSHIP
int main (int argc, char ** argv) {

    TSS_HCONTEXT        hContext;
    TSS_RESULT          result;
    TSS_HTPM            hTPM;
	TSS_HPOLICY			hPolicy;	
	TSS_HPOLICY			hPolicy1;
	char				secret_owner[50] = {0};
	char				secret_srk[50] = {0};
	int					secret_owner_len = 0;
	int					secret_srk_len = 0;
	TSS_FLAG			keyFlags = TSS_KEY_TSP_SRK | TSS_KEY_AUTHORIZATION;
    TSS_HKEY			hSRK;

    TSS_UUID SRK_UUID = TSS_UUID_SRK;

	/* Be careful with scanf!! Buffer Overflow!!! :-) */
	printf("Insert the TPM Owner password: ");
	scanf("%s", secret_owner);
	secret_owner_len = strlen(secret_owner);

	printf("Insert the TPM SRK password: ");
	scanf("%s", secret_srk);
	secret_srk_len = strlen(secret_srk);

    result = Tspi_Context_Create( &hContext );
    if ( result != TSS_SUCCESS )
    {
        printf( "ERR - Tspi_Context_Create");
        Tspi_Context_FreeMemory( hContext, NULL );
        Tspi_Context_Close( hContext );
        exit (ERR);
    }

    result = Tspi_Context_Connect( hContext, NULL );
    if ( result != TSS_SUCCESS )
    {
        printf( "ERR - Tspi_Context_Connect");
        Tspi_Context_FreeMemory( hContext, NULL );
        Tspi_Context_Close( hContext );
        exit (ERR);
    }
    
    result = Tspi_Context_GetTpmObject( hContext, &hTPM );
    if ( result != TSS_SUCCESS )
    {
        printf( "ERR - Tspi_Context_GetTpmObject");
        Tspi_Context_FreeMemory( hContext, NULL );
        Tspi_Context_Close( hContext );
        exit (ERR);
    }

	result = Tspi_GetPolicyObject(hTPM, TSS_POLICY_USAGE, &hPolicy);
	if (result != TSS_SUCCESS) {
        printf( "ERR - Tspi_GetPolicyObject");
        Tspi_Context_FreeMemory( hContext, NULL );
        Tspi_Context_Close( hContext );
        exit (ERR);
	}

	result = Tspi_Policy_SetSecret(hPolicy, TSS_SECRET_MODE_PLAIN,
			secret_owner_len, (BYTE*)secret_owner);
	if (result != TSS_SUCCESS) {
        printf( "ERR - Tspi_Policy_SetSecret");
        Tspi_Context_FreeMemory( hContext, NULL );
        Tspi_Context_Close( hContext );
        exit (ERR);
	}

	result = Tspi_Context_CreateObject(hContext,
			TSS_OBJECT_TYPE_RSAKEY, keyFlags, &hSRK);
	if (result != TSS_SUCCESS) {
		printf( "ERR - Tspi_Context_CreateObject");
        Tspi_Context_FreeMemory( hContext, NULL );
        Tspi_Context_Close( hContext );
        exit (ERR);
	}

	result = Tspi_Context_CreateObject(hContext,
			TSS_OBJECT_TYPE_POLICY, TSS_POLICY_USAGE, &hPolicy1);
	if (result != TSS_SUCCESS) {
		printf( "ERR - Tspi_Context_CreateObject");
        Tspi_Context_FreeMemory( hContext, NULL );
        Tspi_Context_Close( hContext );
        exit (ERR);
	}

	result = Tspi_Policy_SetSecret(hPolicy1, TSS_SECRET_MODE_PLAIN,
		secret_srk_len, (BYTE *)secret_srk);
	if (result != TSS_SUCCESS) {
		printf( "ERR - Tspi_Policy_SetSecret");
        Tspi_Context_FreeMemory( hContext, NULL );
        Tspi_Context_Close( hContext );
        exit (ERR);
	}

	result = Tspi_Policy_AssignToObject(hPolicy1, hSRK);
	if (result != TSS_SUCCESS) {
		printf( "ERR - Tspi_Policy_AssignToObject");
        Tspi_Context_FreeMemory( hContext, NULL );
        Tspi_Context_Close( hContext );
        exit (ERR);
	}

	result = Tspi_TPM_TakeOwnership(hTPM, hSRK, (TSS_HKEY)NULL);
	if (result != TSS_SUCCESS) {
		printf( "ERR - Tspi_TPM_TakeOwnership or Ownership already taken");
        goto end;
	}

end:
	Tspi_Context_CloseObject(hContext,hTPM);
	Tspi_Context_CloseObject(hContext,hSRK);
	Tspi_Context_CloseObject(hContext,hPolicy);
	Tspi_Context_CloseObject(hContext,hPolicy1);
	Tspi_Context_Close(hContext);
}
#endif
