/***************************************************************************
 *                                  _   _ ____  _
 *  Project                     ___| | | |  _ \| |
 *                             / __| | | | |_) | |
 *                            | (__| |_| |  _ <| |___
 *                             \___|\___/|_| \_\_____|
 *
 * Copyright (C) 1998 - 2009, Daniel Stenberg, <daniel@haxx.se>, et al.
 *
 * This software is licensed as described in the file COPYING, which
 * you should have received as part of this distribution. The terms
 * are also available at http://curl.haxx.se/docs/copyright.html.
 *
 * You may opt to use, copy, modify, merge, publish, distribute and/or sell
 * copies of the Software, and permit persons to whom the Software is
 * furnished to do so, under the terms of the COPYING file.
 *
 * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
 * KIND, either express or implied.
 *
 ***************************************************************************/

#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <time.h>
#include <fcntl.h>
#ifdef __MINGW32__
#include <userenv.h>
#else
#include <Userenv.h>
#include <direct.h>
#endif /* __MINGW32__ */

#define trousers_log_file "C://Trousers/tcsd_log.txt"
#define MAX_LINE_LEN 512
#define HEADER_LEN 20

#undef snprintf
#define snprintf _snprintf

char *trousers_getUserDir(void)			//WARNING: code from PhysicsFS
{
	DWORD psize = 100;
	char dummy[100];
	BOOL rc = 0;
	HANDLE processHandle;
	HANDLE accessToken = NULL;
	char *userDir = NULL;

	processHandle = GetCurrentProcess();
	if (OpenProcessToken(processHandle, TOKEN_QUERY, &accessToken)) {
		rc = GetUserProfileDirectory(accessToken, dummy, &psize);
		if(!rc)
			return NULL;

		userDir = strdup(dummy);

		CloseHandle(processHandle);
	}
	return userDir;
}


/* Private pseudo-random number seed. Unsigned integer >= 32bit. Threads
   mutual exclusion is not implemented to acess it since we do not require
   high quality random numbers (only used in form boudary generation). */

static unsigned int randseed;

/* Pseudo-random number support. */

unsigned int Curl_rand(void)
{
	unsigned int r;
	/* Return an unsigned 32-bit pseudo-random number. */
	r = randseed = randseed * 1103515245 + 12345;
	return (r << 16) | ((r >> 16) & 0xFFFF);
}

void Curl_srand(void)
{
	/* Randomize pseudo-random number sequence. */

	randseed = (unsigned int) time(NULL);
	Curl_rand();
	Curl_rand();
	Curl_rand();
}

void trousers_setenv(const char *name, const char *value, int overwrite) {
	int len = 0;
	char *str = NULL;

	if (getenv(name) != NULL && overwrite == 0)
		return;

	len = strlen(name) + 1 + strlen(value) + 1 + 1;
	str = malloc(len);
	if (str == NULL)
		return;

	snprintf(str, len, "%s=%s", name, value);
	putenv(str);
	free(str);
}

void trousers_log(const char *layer, const char *fmt, ...)
{
	int fh;
	va_list ap;
	char *line = NULL;
	int offset = 0;

	time_t tv;
	struct tm t;
	time(&tv);
	memcpy(&t, localtime(&tv), sizeof(t));
	va_start(ap, fmt);

	fh = open(trousers_log_file, O_CREAT | O_WRONLY | O_APPEND | O_BINARY, 0600);
	if (fh >=0) {
		line = (char *)malloc(MAX_LINE_LEN);
		if (line == NULL)
			return;

		memset(line, 0, MAX_LINE_LEN);
		offset += snprintf(line, MAX_LINE_LEN, "%04d-%02d-%02d %02d:%02d:%02d ",
			t.tm_year + 1900, t.tm_mon + 1, t.tm_mday,
			t.tm_hour, t.tm_min, t.tm_sec);
		offset += snprintf(line + offset, MAX_LINE_LEN - offset, "-%s- ", layer);
		offset += vsnprintf(line + offset, MAX_LINE_LEN - offset, fmt, ap);
		write(fh, line, offset);
		close(fh);
		free(line);
	}
	va_end(ap);
}
